package com.lzmispt.data.resource.service.metadata.impl;

package .tmp;

import com.google.common.base.Objects;
import com.lzmispt.data.center.entity.dto.standard.metadata.GetDirByUseTypeDTO;
import com.lzmispt.data.center.entity.vo.resource.metadata.DirectoryTreeVO;
import com.lzmispt.data.center.entity.vo.resource.metadata.MetadataDirectoryVO;
import com.lzmispt.data.center.entity.vo.standard.metadata.EntityPublishType;
import com.lzmispt.data.resource.client.DataStandardClient;
import com.lzmispt.data.resource.dao.metadata.IMetadataDao;
import com.lzmispt.data.resource.entity.dto.metadata.GetDirectoryTreeByEntityIdsDTO;
import com.lzmispt.data.resource.entity.dto.metadata.GetDirectoryTreeByPubStateDTO;
import com.lzmispt.data.resource.entity.po.metadata.MetadataDirectoryPO;
import com.lzmispt.data.resource.entity.vo.metadata.DirectoryTreeBaseVO;
import com.lzmispt.data.resource.service.metadata.impl.DataResourceManager;
import com.lzmispt.dcbase.ServiceUtil;
import com.lzmispt.framework.entity.entity.ResultEntity;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

@Service
public class StandardDataResourceImpl extends DataResourceManager {
  @Autowired
  private DataStandardClient standardClient;
  
  public String getDefaultCode() {
    return "Standard";
  }
  
  public ResultEntity<List<DirectoryTreeBaseVO>> getDirectoryTreeByEntityIds(GetDirectoryTreeByEntityIdsDTO getDirectoryTreeByEntityIdsDTO) {
    Map<Long, Long> entityIdAndEntityId = (Map<Long, Long>)this.standardClient.getDirectoryIdsByEntityIds(getDirectoryTreeByEntityIdsDTO.getEntityIds(), getDirectoryTreeByEntityIdsDTO.getDataResourceCategory(), getDefaultCode(), getDirectoryTreeByEntityIdsDTO.getTokenId()).getObj();
    List<Long> directoryIds = (List<Long>)entityIdAndEntityId.values().stream().collect(Collectors.toList());
    List<EntityPublishType> entityPublishTypes = (List<EntityPublishType>)this.standardClient.getEntityPublishType(getDirectoryTreeByEntityIdsDTO.getPlatCode(), getDirectoryTreeByEntityIdsDTO.getTokenId(), getDirectoryTreeByEntityIdsDTO.getDataResourceCategory(), getDirectoryTreeByEntityIdsDTO.getEntityIds()).getObj();
    Map<Long, EntityPublishType> entityPublishTypeMap = (Map<Long, EntityPublishType>)entityPublishTypes.stream().collect(Collectors.toMap(EntityPublishType::getDirectoryId, Function.identity()));
    List<DirectoryTreeBaseVO> top = new LinkedList<>();
    List<MetadataDirectoryPO> metadataDirectories = ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).selectRecursionParentDirectoryByNameAndDirIds(getDirectoryTreeByEntityIdsDTO.getName(), directoryIds);
    List<DirectoryTreeBaseVO> allGetPubEntityTreeList = new LinkedList<>();
    for (MetadataDirectoryPO metadataDirectory : metadataDirectories) {
      DirectoryTreeBaseVO directoryTreeVO = new DirectoryTreeBaseVO();
      BeanUtils.copyProperties(metadataDirectory, directoryTreeVO);
      allGetPubEntityTreeList.add(directoryTreeVO);
    } 
    top = (List<DirectoryTreeBaseVO>)allGetPubEntityTreeList.stream().filter(s -> (s.getParentDirectoryId() == null)).sorted(Comparator.comparingInt(DirectoryTreeBaseVO::getDirectoryOrder)).collect(Collectors.toList());
    assemblePubEntityTree(top, allGetPubEntityTreeList, entityPublishTypeMap);
    return ResultEntity.successMessage(top);
  }
  
  public ResultEntity<List<DirectoryTreeVO>> getDirectoryTreeByPubState(GetDirectoryTreeByPubStateDTO getDirectoryTreeByPubStateDTO) {
    List<DirectoryTreeVO> top = new LinkedList<>();
    GetDirByUseTypeDTO getDirByUseTypeDTO = new GetDirByUseTypeDTO();
    getDirByUseTypeDTO.setUseType(getDirectoryTreeByPubStateDTO.getPubState());
    getDirByUseTypeDTO.setDataCategory(getDirectoryTreeByPubStateDTO.getDataResourceCategory());
    List<Long> directoryIds = ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).getAllDirectoryTableIds();
    getDirByUseTypeDTO.setDirIds(directoryIds);
    if (CollectionUtils.isEmpty(directoryIds))
      return ResultEntity.successMessage(null); 
    List<Long> filteredDirectoryIds = (List<Long>)this.standardClient.getDirIdByUseType(getDefaultCode(), getDirectoryTreeByPubStateDTO.getTokenId(), getDirByUseTypeDTO).getObj();
    if (CollectionUtils.isEmpty(filteredDirectoryIds))
      return ResultEntity.successMessage(null); 
    List<MetadataDirectoryPO> metadataDirectories = ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).selectRecursionParentDirectoryByNameAndDirIds(getDirectoryTreeByPubStateDTO.getName(), filteredDirectoryIds);
    List<DirectoryTreeVO> allDirectoryTreeList = new ArrayList<>();
    for (MetadataDirectoryPO metadataDirectoryPO : metadataDirectories) {
      DirectoryTreeVO directoryTreeVO = new DirectoryTreeVO();
      BeanUtils.copyProperties(metadataDirectoryPO, directoryTreeVO);
      allDirectoryTreeList.add(directoryTreeVO);
    } 
    top = (List<DirectoryTreeVO>)allDirectoryTreeList.stream().filter(s -> (s.getParentDirectoryId() == null)).sorted(Comparator.comparingInt(MetadataDirectoryVO::getDirectoryOrder)).collect(Collectors.toList());
    assembleDirectoryTreeVO(top, allDirectoryTreeList);
    return ResultEntity.successMessage(top);
  }
  
  private void assembleDirectoryTreeVO(List<DirectoryTreeVO> curDirectoryTreeList, List<DirectoryTreeVO> allDirectoryTreeList) {
    if (!CollectionUtils.isEmpty(curDirectoryTreeList))
      for (DirectoryTreeVO directoryTreeVO : curDirectoryTreeList) {
        List<DirectoryTreeVO> child = (List<DirectoryTreeVO>)allDirectoryTreeList.stream().filter(s -> Objects.equal(directoryTreeVO.getDirectoryId(), s.getParentDirectoryId())).sorted(Comparator.comparingInt(MetadataDirectoryVO::getDirectoryOrder)).collect(Collectors.toList());
        directoryTreeVO.setChildren(child);
        assembleDirectoryTreeVO(child, allDirectoryTreeList);
      }  
  }
  
  private void assemblePubEntityTree(List<DirectoryTreeBaseVO> curGetPubEntityTreeList, List<DirectoryTreeBaseVO> allGetPubEntityTreeList, Map<Long, EntityPublishType> entityPublishTypeMap) {
    if (!CollectionUtils.isEmpty(curGetPubEntityTreeList))
      for (DirectoryTreeBaseVO getPubEntityTree : curGetPubEntityTreeList) {
        if (entityPublishTypeMap.containsKey(getPubEntityTree.getDirectoryId()))
          getPubEntityTree.setState(((EntityPublishType)entityPublishTypeMap.get(getPubEntityTree.getDirectoryId())).getPublishType()); 
        List<DirectoryTreeBaseVO> child = (List<DirectoryTreeBaseVO>)allGetPubEntityTreeList.stream().filter(s -> Objects.equal(getPubEntityTree.getDirectoryId(), s.getParentDirectoryId())).sorted(Comparator.comparingInt(DirectoryTreeBaseVO::getDirectoryOrder)).collect(Collectors.toList());
        getPubEntityTree.setChildren(child);
        assemblePubEntityTree(child, allGetPubEntityTreeList, entityPublishTypeMap);
      }  
  }
}
